#include "types.hh"

bool     OptimizeChangeLog   = true;
unsigned AnimationBlurLength = 0;
unsigned LoopingLogLength    = 16;
int      FirstLastLength     = 16;

#include "pixel.hh"
#include "pixels.hh"

unsigned long pixelmethods_result = 1ul << pm_MostUsedPixel;
enum PixelMethod bgmethod = pm_MostUsedPixel;

template<typename T>
struct Array256x256of: public Array256x256of_Base
{
public:
    T data[256*256];
public:
    Array256x256of() { }
    virtual ~Array256x256of() { }
public:
    virtual uint32 GetLive(unsigned method, unsigned index, unsigned timer) const FastPixelMethod
    {
        return (data[index].*(T::methods[method]))(timer);
    }

    virtual uint32 GetStatic(unsigned index) const
    {
        return (data[index].*(T::methods[bgmethod]))(0);
    }

    virtual void Set(unsigned index, uint32 p, unsigned timer)
    {
        data[index].set(p, timer);
    }

    virtual unsigned GetPixelSize() const
    {
        return sizeof(T);
    }

    virtual const char* GetPixelSetupName() const
    {
        return T::name;
    }
};

namespace
{
    struct FactoryType
    {
        typedef Array256x256of_Base ObjT;
        ObjT* (*Construct)();
        ObjT* (*Copy)(const ObjT& b);
        void  (*Assign)(ObjT& tgt, const ObjT& b);
    };
    template<typename T>
    struct FactoryMethods
    {
        typedef Array256x256of_Base ObjT;
        typedef Array256x256of<T> ResT;
        static ObjT* Construct()         { return new ResT; }
        static ObjT* Copy(const ObjT& b) { return new ResT ( (const ResT&) b ) ; }
        static void Assign(ObjT& tgt, const ObjT& b) { (ResT&) tgt = (const ResT&) b; }

        static const FactoryType data;
    };
    template<typename T>
    const FactoryType FactoryMethods<T>::data =
    {
        FactoryMethods<T>::Construct,
        FactoryMethods<T>::Copy,
        FactoryMethods<T>::Assign
    };

#include "pixelfactory.inc"

    struct Get256x256pixelFactory
    {
        inline const FactoryType* operator-> () const
        {
            return FindFactory( pixelmethods_result | (1ul << bgmethod) );
        }
    };
}

void UncertainPixelVector256x256::init()
{
    /* Construct the type of object determined by the globals "pixelmethod" and "bgmethod" */
    if(!data) data = Get256x256pixelFactory()->Construct();
}

// Copy constructor
UncertainPixelVector256x256::UncertainPixelVector256x256
    (const UncertainPixelVector256x256& b) : data(0)
{
    if(b.data) data = Get256x256pixelFactory()->Copy(*b.data);
}

// Assign operator (create copy of the other vector)
UncertainPixelVector256x256&
    UncertainPixelVector256x256::operator=
        (const UncertainPixelVector256x256& b)
{
    if(!b.data) { delete data; data=0; }
    else if(data) Get256x256pixelFactory()->Assign(*data, *b.data);
    else data = Get256x256pixelFactory()->Copy(*b.data);
    return *this;
}

unsigned GetPixelSizeInBytes()
{
    Array256x256of_Base* p = Get256x256pixelFactory()->Construct();
    unsigned result = p->GetPixelSize();
    delete p;
    return result;
}

const char* GetPixelSetupName()
{
    Array256x256of_Base* p = Get256x256pixelFactory()->Construct();
    const char* result = p->GetPixelSetupName();
    delete p;
    return result;
}
